from typing import List
from typing import TypedDict


class CustomFieldJSON(TypedDict):

    id: int
    name: str
    value: str


class CustomFieldListJSON(TypedDict):

    plan_content_custom_fields: List[CustomFieldJSON]
    requirement_custom_fields: List[CustomFieldJSON]
    step_custom_fields: List[CustomFieldJSON]
    executed_step_custom_fields: List[CustomFieldJSON]



from typing import List
from typing import Optional
from typing import TypedDict

from testlodge.typing.shared import DateTimeStr
from testlodge.typing.shared import Pagination


class RunJSON(TypedDict):

    id: int
    name: str
    incomplete_number: int
    passed_number: int
    skipped_number: int
    failed_number: int
    user_id: Optional[int]
    project_id: int
    executed_plan_id: Optional[int]
    created_at: DateTimeStr
    updated_at: DateTimeStr


class RunListJSON(TypedDict):

    pagination: Pagination
    steps: List[RunJSON]



from types import SimpleNamespace
from typing import List
from typing import Type

import requests
from furl import furl
from furl.furl import furl as Url
from requests import Response
from testlodge.api.base import BaseAPI as API
from testlodge.api.case import CaseAPI
from testlodge.api.custom_field import CustomFieldAPI
from testlodge.api.plan import PlanAPI
from testlodge.api.project import ProjectAPI
from testlodge.api.requirement import RequirementAPI
from testlodge.api.requirement_document import RequirementDocumentAPI
from testlodge.api.run import RunAPI
from testlodge.api.suite import SuiteAPI
from testlodge.api.suite_section import SuiteSectionAPI
from testlodge.api.user import UserAPI


class BaseClient:
    """Represents a client accessing the TestLodge API."""

    def __init__(
        self,
        email: str,
        api_key: str,
        account_id: int,
        apis: List[Type[API]] = None,
    ):

        if apis is None:
            apis = [
                UserAPI,
                CustomFieldAPI,
                ProjectAPI,
                SuiteSectionAPI,
                SuiteAPI,
                CaseAPI,
                PlanAPI,
                RunAPI,
                RequirementAPI,
                RequirementDocumentAPI,
            ]

        self.email: str = email
        self.api_key: str = api_key
        self.account_id: int = account_id

        self.history: List[Response] = []

        # Initialize the APIs
        self.api = SimpleNamespace()
        for api in apis:
            api_instance = api(client=self)
            setattr(self.api, api.name, api_instance)

    @property
    def base_url(self) -> Url:

        return furl(f'https://api.testlodge.com/v1/account/{self.account_id}')

    def _request(self, method: str, url: str, *args, **kwargs) -> Response:
        """Wrap requests.request to add handlers for response,
        logging, etc."""

        response = requests.request(
            method=method,
            url=url,
            auth=(self.email, self.api_key),
            *args,  # type: ignore
            **kwargs,  # type: ignore
        )

        self.history.append(response)

        return response



from enum import auto
from enum import IntEnum
from typing import Optional

from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.typing.run import RunJSON
from testlodge.typing.run import RunListJSON


class SortRunOrder(IntEnum):
    """Method to sort runs by."""

    CREATED_AT = auto()
    NAME = auto()


class Status(IntEnum):
    """Run Status"""

    NOT_STARTED = auto()
    IN_PROGRESS = auto()
    COMPLETE = auto()


class RunAPI(BaseAPI):
    """API for test runs.

    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'run'

    def _list(
        self,
        *,
        project_id: int,
        page: int = 1,
        user_id: int = None,
        status: Optional[Status] = None,
        order: SortRunOrder = SortRunOrder.CREATED_AT,
    ) -> RunListJSON:
        """Paginated list of all runs in a project.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        page: int, default=1
            Default: 1
            The number of the page to return.
        user_id: int, optional
            Filter by user.
        status: Status, optional, default=None
            Default: None
            Status of the run.
        order: SortRunOrder, default=SortRunOrder.CREATED_AT
            Default: SortRunOrder.CREATED_AT
            Method to sort the list of runs.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/runs.json'
        )
        params: dict = {}
        if page != 1:
            params['page'] = page
        if order != SortRunOrder.CREATED_AT:
            params['order'] = int(order)
        if status is not None:
            params['status'] = int(status)
        if user_id is not None:
            params['user_id'] = user_id

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        run_list: RunListJSON = response.json()

        return run_list

    def _show(
        self,
        *,
        project_id: int,
        run_id: int,
    ) -> RunJSON:
        """Get the details for a run.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        run_id: int
            The ID of the run.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/runs/{run_id}.json'
        )

        response: Response = self.client._request(
            method=method,
            url=url,
        )
        run_json: RunJSON = response.json()

        return run_json

    def _create(
        self,
        *,
        project_id: int,
        run: RunJSON,
    ) -> RunJSON:
        """Create a run.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        run: RunJSON

            name: str
                Run name.
            step_ids: list[int]
                Either step_ids or suite_ids is required.
                Associated case IDs.
            suite_ids: list[int]
                Either step_ids or suite_ids is required.
                Associated suite IDs.
            plan_id: int, optional
                Associated test plan.
            user_id: int, optional
                Associated user.
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' '/runs.json'
        )

        data = dict(run=run)

        response: Response = self.client._request(
            method=method, url=url, json=data
        )
        run_json: RunJSON = response.json()

        return run_json

    def _update(
        self,
        *,
        project_id: int,
        run_id: int,
        run: RunJSON,
    ) -> RunJSON:
        """Update a run.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        run_id: int
            The ID of the run.
        run: RunJSON

            name: str
                Name of the run.
            plan_id: int
                Associated test plan.
            user_id: int
                Associated user.
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/runs/{run_id}.json'
        )
        data = dict(run=run)

        response: Response = self.client._request(
            method=method,
            url=url,
            json=data,
        )
        run_json: RunJSON = response.json()

        return run_json

    def _delete(
        self,
        *,
        project_id: int,
        run_id: int,
    ) -> None:
        """Delete a run.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        run_id: int
            The ID of the run.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/runs/{run_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)

        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None



from testlodge.api.user import UserAPI
from testlodge.client.base import BaseClient
from testlodge.typing.user import UserListJSON


class UserClient(BaseClient):
    def list_user_json(self, page: int = 1) -> UserListJSON:
        return getattr(self.api, UserAPI.name)._list(page)



from testlodge.typing.case import CaseJSON
from testlodge.typing.case import CaseListJSON
from testlodge.typing.custom_field import CustomFieldJSON
from testlodge.typing.custom_field import CustomFieldListJSON
from testlodge.typing.plan import PlanJSON
from testlodge.typing.plan import PlanListJSON
from testlodge.typing.project import ProjectJSON
from testlodge.typing.project import ProjectListJSON
from testlodge.typing.requirement import RequirementJSON
from testlodge.typing.requirement import RequirementListJSON
from testlodge.typing.requirement_document import RequirementDocumentJSON
from testlodge.typing.requirement_document import RequirementDocumentListJSON
from testlodge.typing.run import RunJSON
from testlodge.typing.run import RunListJSON
from testlodge.typing.shared import DateTimeStr
from testlodge.typing.shared import Pagination
from testlodge.typing.suite import SuiteJSON
from testlodge.typing.suite import SuiteListJSON
from testlodge.typing.suite_section import SuiteSectionJSON
from testlodge.typing.suite_section import SuiteSectionListJSON
from testlodge.typing.user import UserJSON
from testlodge.typing.user import UserListJSON


__all__ = [
    'CaseJSON',
    'CaseListJSON',
    'CustomFieldJSON',
    'CustomFieldListJSON',
    'PlanJSON',
    'PlanListJSON',
    'ProjectJSON',
    'ProjectListJSON',
    'RequirementJSON',
    'RequirementListJSON',
    'RequirementDocumentJSON',
    'RequirementDocumentListJSON',
    'RunJSON',
    'RunListJSON',
    'SuiteJSON',
    'SuiteListJSON',
    'SuiteSectionJSON',
    'SuiteSectionListJSON',
    'UserJSON',
    'UserListJSON',
    'DateTimeStr',
    'Pagination',
]



from enum import auto
from enum import IntEnum

from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.typing.plan import PlanJSON
from testlodge.typing.plan import PlanListJSON


class SortPlanOrder(IntEnum):
    """Method to sort by."""

    CREATED_AT = auto()
    UPDATED_AT = auto()
    NAME = auto()


class PlanAPI(BaseAPI):
    """API for test plans.

    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'plan'

    def _list(
        self,
        *,
        project_id: int,
        page: int = 1,
        order: SortPlanOrder = SortPlanOrder.CREATED_AT,
    ) -> PlanListJSON:
        """Paginated list of all plans in a project.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        page: int, default=1
            Default: 1
            The number of the page to return.
        order: SortPlanOrder, default=SortPlanOrder.CREATED_AT
            Default: SortPlanOrder.CREATED_AT
            Method to sort the list.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' 'plans.json'
        )
        params: dict = {}
        if page != 1:
            params['page'] = page
        if order != SortPlanOrder.CREATED_AT:
            params['order'] = int(order)

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        plan_list: PlanListJSON = response.json()

        return plan_list

    def _show(
        self,
        *,
        project_id: int,
        plan_id: int,
    ) -> PlanJSON:
        """Get the details for a plan.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        plan_id: int
            The ID of the plan.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/plans/{plan_id}.json'
        )

        response: Response = self.client._request(
            method=method,
            url=url,
        )
        plan_json: PlanJSON = response.json()

        return plan_json

    def _create(
        self,
        *,
        project_id: int,
        plan: PlanJSON,
    ) -> PlanJSON:
        """Create a plan.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        plan: PlanJSON

            name: str
                Name of the plan
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' 'plans.json'
        )

        data = dict(plan=plan)

        response: Response = self.client._request(
            method=method, url=url, json=data
        )
        plan_json: PlanJSON = response.json()

        return plan_json

    def _update(
        self,
        *,
        project_id: int,
        plan_id: int,
        plan: PlanJSON,
    ) -> PlanJSON:
        """Update a plan.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        plan_id: int
            The ID of the plan.
        plan: PlanJSON

            name: str
                Name of the plan
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/plans/{plan_id}.json'
        )

        data = dict(plan=plan)

        response: Response = self.client._request(
            method=method,
            url=url,
            json=data,
        )
        plan_json: PlanJSON = response.json()

        return plan_json

    def _delete(self, *, project_id: int, plan_id: int) -> None:
        """Delete a plan.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        plan_id: int
            The ID of the plan.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/plans/{plan_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)

        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None



from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.typing.user import UserListJSON


class UserAPI(BaseAPI):
    """API for users.

    Endpoints
    ---------
    * List
    """

    name: str = 'user'

    def _list(
        self,
        page: int = 1,
    ) -> UserListJSON:
        """Paginated list of all users in an account.

        Parameters
        ----------
        page: int, default=1
            Default: 1
            Page to return.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath('/users.json')
        if page != 1:
            params = {'page': page}
        else:
            params = {}

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        user_list: UserListJSON = response.json()

        return user_list



from typing import List
from typing import TypedDict

from testlodge.typing.shared import DateTimeStr
from testlodge.typing.shared import Pagination


class ProjectJSON(TypedDict):

    id: int
    name: str
    description: str
    issue_tracker_credential_id: int
    issue_tracker_project_id: str
    created_at: DateTimeStr
    updated_at: DateTimeStr


class ProjectListJSON(TypedDict):

    pagination: Pagination
    steps: List[ProjectJSON]



from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.typing.suite_section import SuiteSectionJSON
from testlodge.typing.suite_section import SuiteSectionListJSON


class SuiteSectionAPI(BaseAPI):
    """API for suite sections.


    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'suite_section'

    def _list(
        self, *, project_id: int, suite_id: int, page: int = 1
    ) -> SuiteSectionListJSON:
        """Paginated list of all suite sections inside a suite.

        Parameters
        ----------
        project_id: Identifier
            The ID of the project.
        suite_id: Identifier
            The ID of the suite.
        page: int, default=1
            Default: 1
            The number of the page to return.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            '/suite_sections.json'
        )
        if page != 1:
            params = {'page': page}
        else:
            params = {}

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        data: SuiteSectionListJSON = response.json()

        return data

    def _show(
        self,
        *,
        project_id: int,
        suite_id: int,
        suite_section_id: int,
    ) -> SuiteSectionJSON:
        """Get the details for a _suite section_.

        Parameters
        ----------
        project_id: Identifier
            The ID of the project.
        suite_id: Identifier
            The ID of the suite.
        suite_section_id: Identifier
            The ID of the suite section.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            f'/suite_sections/'
            f'{suite_section_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)
        data: SuiteSectionJSON = response.json()

        return data

    def _create(
        self,
        *,
        project_id: int,
        suite_id: int,
        suite_section: SuiteSectionJSON,
    ) -> SuiteSectionJSON:
        """Create a new _suite section_.

        Parameters
        ----------
        project_id: Identifier
            The ID of the project.
        suite_id: Identifier
            The ID of the suite.
        suite_section: SuiteSectionJSON
            The suite section.
            Example:
                {
                    'title': 'Section 1'
                }
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            f'/suite_sections.json'
        )

        response: Response = self.client._request(
            method=method, url=url, json=suite_section
        )
        new_suite_section: SuiteSectionJSON = response.json()

        return new_suite_section

    def _update(
        self,
        *,
        project_id: int,
        suite_id: int,
        suite_section_id: int,
        suite_section: SuiteSectionJSON,
    ) -> SuiteSectionJSON:
        """Update a _suite section_.

        Parameters
        ----------
        project_id: Identifier
            The ID of the project.
        suite_id: Identifier
            The ID of the suite.
        suite_section_id: Identifier
            The ID of the suite section.
        suite_section: SuiteSectionJSON
            The suite section.
            Example:
                {
                    'title': 'Section 1'
                }
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            f'/suite_sections/{suite_section_id}.json'
        )

        response: Response = self.client._request(
            method=method, url=url, json=suite_section
        )
        updated_suite_section: SuiteSectionJSON = response.json()

        return updated_suite_section

    def _delete(
        self,
        *,
        project_id: int,
        suite_id: int,
        suite_section_id: int,
    ) -> None:
        """Delete a _suite section_.

        Parameters
        ----------
        project_id: Identifier
            The ID of the project.
        suite_id: Identifier
            The ID of the suite.
        suite_section_id: Identifier
            The ID of the suite section.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            f'/suite_sections/{suite_section_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)
        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None



from typing import Optional

from testlodge.api.project import ProjectAPI
from testlodge.client.base import BaseClient
from testlodge.typing.project import ProjectJSON
from testlodge.typing.project import ProjectListJSON


class ProjectClient(BaseClient):
    def list_project_json(self, page: int = 1) -> ProjectListJSON:
        return getattr(self.api, ProjectAPI.name)._list(page)

    def show_project_json(self, *, project_id: int) -> ProjectJSON:
        return getattr(self.api, ProjectAPI.name)._show(project_id=project_id)

    def create_project_json(
        self,
        name: str,
        *,
        description: Optional[str] = None,
        issue_tracker_credential_id: Optional[int] = None,
        issue_tracker_project_id: Optional[str] = None,
    ) -> ProjectJSON:
        return getattr(self.api, ProjectAPI.name)._create(
            name=name,
            description=description,
            issue_tracker_credential_id=issue_tracker_credential_id,
            issue_tracker_project_id=issue_tracker_project_id,
        )

    def update_project_json(
        self,
        *,
        project_id: int,
        project: ProjectJSON,
    ) -> ProjectJSON:
        return getattr(self.api, ProjectAPI.name)._update(
            project_id=project_id,
            project=project,
        )

    def delete_project_json(
        self,
        *,
        project_id: int,
    ) -> ProjectJSON:
        return getattr(self.api, ProjectAPI.name)._delete(
            project_id=project_id,
        )



from typing import List
from typing import TypedDict

from testlodge.typing.shared import DateTimeStr
from testlodge.typing.shared import Pagination


class RequirementDocumentJSON(TypedDict):

    id: int
    title: str
    should_version: bool
    project_id: int
    created_at: DateTimeStr
    updated_at: DateTimeStr


class RequirementDocumentListJSON(TypedDict):

    pagination: Pagination
    requirement_documents: List[RequirementDocumentJSON]



from enum import auto
from enum import IntEnum
from typing import List
from typing import Optional

from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.typing.requirement_document import RequirementDocumentJSON
from testlodge.typing.requirement_document import RequirementDocumentListJSON


class SortRequirementDocumentOrder(IntEnum):
    """Method to sort requirement documents by."""

    CREATED_AT = auto()
    UPDATED_AT = auto()
    TITLE = auto()


class RequirementDocumentAPI(BaseAPI):
    """API for requirement documents.

    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'requirement_document'

    def _list(
        self,
        *,
        project_id: int,
        page: int = 1,
        order: SortRequirementDocumentOrder = SortRequirementDocumentOrder.CREATED_AT,
    ) -> RequirementDocumentListJSON:
        """Paginated list of all requirement documents in a project.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        page: int, default=1
            Default: 1
            The number of the page to return.
        order: SortRequirementDocumentOrder, default=SortRequirementDocumentOrder.CREATED_AT
            Default: SortOrder.CREATED_AT
            Method to sort the list of requirement documents.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/requirement_documents.json'
        )
        params: dict = {}
        if page != 1:
            params['page'] = page
        if order != SortRequirementDocumentOrder.CREATED_AT:
            params['order'] = int(order)

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        requirement_documents_list: RequirementDocumentListJSON = (
            response.json()
        )

        return requirement_documents_list

    def _show(
        self,
        *,
        project_id: int,
        requirement_document_id: int,
    ) -> RequirementDocumentJSON:
        """Get the details for a requirement documents.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/requirement_documents/{requirement_document_id}.json'
        )

        response: Response = self.client._request(
            method=method,
            url=url,
        )
        requirement_document_json: RequirementDocumentJSON = response.json()

        return requirement_document_json

    def _create(
        self,
        *,
        project_id: int,
        requirement_document: RequirementDocumentJSON,
    ) -> RequirementDocumentJSON:
        """Create a requirement document.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document: RequirementDocumentJSON

            title: str
                Title of the requirement.
            should_version: bool, default=False
                Default: False
                Should content be versioned when changed.
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/requirement_documents.json.'
        )

        data = dict(requirement_document=requirement_document)

        response: Response = self.client._request(
            method=method, url=url, json=data
        )
        requirement_json: RequirementDocumentJSON = response.json()

        return requirement_json

    def _update(
        self,
        *,
        project_id: int,
        requirement_document_id: int,
        requirement_document: RequirementDocumentJSON,
    ) -> RequirementDocumentJSON:
        """Update a requirement.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        requirement_document: RequirementDocumentJSON

            title: str
                Title of the requirement.
            should_version: bool, default=False
                Default: False
                Should content be versioned when changed.
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/requirement_documents/{requirement_document_id}.json'
        )
        data = dict(requirement_document=requirement_document)

        response: Response = self.client._request(
            method=method,
            url=url,
            json=data,
        )
        requirement_json: RequirementDocumentJSON = response.json()

        return requirement_json

    def _delete(
        self,
        *,
        project_id: int,
        requirement_document_id: int,
    ) -> None:
        """Delete a requirement document.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/requirement_documents/{requirement_document_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)

        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None



from testlodge.api.custom_field import CustomFieldAPI
from testlodge.client.base import BaseClient
from testlodge.typing.custom_field import CustomFieldJSON
from testlodge.typing.custom_field import CustomFieldListJSON


class CustomFieldClient(BaseClient):
    def list_custom_field_json(
        self,
        *,
        project_id: int,
    ) -> CustomFieldListJSON:
        return getattr(self.api, CustomFieldAPI.name)._list(project_id)

    def create_custom_field_json(self) -> CustomFieldJSON:
        raise NotImplementedError("This action is not supported.")



from typing import Dict
from typing import Optional

from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.typing.case import CaseJSON
from testlodge.typing.case import CaseListJSON


class CaseAPI(BaseAPI):
    """API for test cases.

    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'case'

    def _list(
        self,
        *,
        project_id: int,
        suite_id: int,
        suite_section_id: int,
        page: int = 1,
    ) -> CaseListJSON:
        """Paginated list of all cases inside a suite section.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        suite_section_id: int
            The ID of the suite section.
        page: int, default=1
            Default: 1
            The number of the page to return.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            f'/suite_sections/{suite_section_id}/steps.json'
        )
        if page != 1:
            params = {'page': page}
        else:
            params = {}

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        case_list: CaseListJSON = response.json()

        return case_list

    def _show(
        self,
        *,
        project_id: int,
        suite_id: int,
        case_id: int,
        include: Optional[Dict[str, str]] = None,
    ) -> CaseJSON:
        """Get the details for a _case_.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        case_id: int
            The ID of the test case.
        include: dict[str], optional
            An array of strings, representing the additional options to include
            in the response.

            steps_uploads: str
                Any file that has been uploaded and associated with the test
                (urls in the response will only be available for ~30 seconds).
            requirements: list[RequirementJSON]
                Any requirements that have been associated to the case.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            f'/steps/{case_id}.json'
        )

        if include is not None:
            raise NotImplementedError('Not implemented yet.')
        else:
            include = {}

        response: Response = self.client._request(
            method=method, url=url, params=include
        )
        case_json: CaseJSON = response.json()

        return case_json

    def _create(
        self,
        *,
        project_id: int,
        suite_id: int,
        step: CaseJSON,
        suite_section_id: Optional[int] = None,
    ) -> CaseJSON:
        """Create a test case.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        step: dict[CaseJSON]

            title: str
            description: str, optional
            test_steps: str, optional
            expected_result: str, optional
            custom_fields: list[CustomField], optional
                Custom fields of the test case.

        suite_section_id: int, optional
            Default: The top suite section in the test suite, or creates one if
            none exists.
            The ID of the suite section.
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/suites/{suite_id}/steps.json'
        )

        if suite_section_id is not None:
            data = {'suite_section_id': suite_section_id, 'step': step}
        else:
            data = {'step': step}

        response: Response = self.client._request(
            method=method, url=url, json=data
        )
        case_json: CaseJSON = response.json()

        return case_json

    def _update(
        self,
        *,
        project_id: int,
        suite_id: int,
        case_id: int,
        step: CaseJSON,
    ) -> CaseJSON:
        """Update a test case.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        case_id: int
            The ID of the test case.
        step: CaseJSON

            title: str
            description: str, optional
            test_steps: str, optional
            expected_result: str, optional
            custom_fields: list[CustomField], optional
                Custom fields of the test case.
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}/suites/{suite_id}/steps/{case_id}.json'
        )

        response: Response = self.client._request(
            method=method, url=url, json=step
        )
        case_json: CaseJSON = response.json()

        return case_json

    def _delete(self, *, project_id: int, suite_id: int, case_id: int) -> None:
        """Delete a test case.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        case_id: int
            The ID of the test case.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/suites/{suite_id}'
            f'/steps/{case_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)

        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None



from enum import auto
from enum import IntEnum

from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.typing.suite import SuiteJSON
from testlodge.typing.suite import SuiteListJSON


class SortSuiteOrder(IntEnum):
    """Method to sort by."""

    CREATED_AT = auto()
    UPDATED_AT = auto()
    NAME = auto()


class SuiteAPI(BaseAPI):
    """API for test suites.

    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'suite'

    def _list(
        self,
        *,
        project_id: int,
        page: int = 1,
        order: SortSuiteOrder = SortSuiteOrder.CREATED_AT,
    ) -> SuiteListJSON:
        """Paginated list of all suites in a project.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        page: int, default=1
            Default: 1
            The number of the page to return.
        order: SortSuiteOrder, default=SortSuiteOrder.CREATED_AT
            Default: SortSuiteOrder.CREATED_AT
            Method to sort the list of suites.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' '/suites.json'
        )
        params: dict = {}
        if page != 1:
            params['page'] = page
        if order != SortSuiteOrder.CREATED_AT:
            params['order'] = int(order)

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        suite_list: SuiteListJSON = response.json()

        return suite_list

    def _show(
        self,
        *,
        project_id: int,
        suite_id: int,
    ) -> SuiteJSON:
        """Get the details for a suite.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/suites/{suite_id}.json'
        )

        response: Response = self.client._request(
            method=method,
            url=url,
        )
        suite_json: SuiteJSON = response.json()

        return suite_json

    def _create(
        self,
        *,
        project_id: int,
        suite: SuiteJSON,
    ) -> SuiteJSON:
        """Create a suite.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite: SuiteJSON

            name: str
                Name of the suite.
            plan_id: int, optional
                Associated test plan.
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' '/suites.json'
        )

        data = dict(suite=suite)

        response: Response = self.client._request(
            method=method, url=url, json=data
        )
        suite_json: SuiteJSON = response.json()

        return suite_json

    def _update(
        self,
        *,
        project_id: int,
        suite_id: int,
        suite: SuiteJSON,
    ) -> SuiteJSON:
        """Update a suite.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        suite: SuiteJSON

            name: str
                Name of the suite.
            plan_id: int
                Associated test plan.
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/suites/{suite_id}.json'
        )
        data = dict(suite=suite)

        response: Response = self.client._request(
            method=method,
            url=url,
            json=data,
        )
        suite_json: SuiteJSON = response.json()

        return suite_json

    def _delete(
        self,
        *,
        project_id: int,
        suite_id: int,
    ) -> None:
        """Delete a suite.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        suite_id: int
            The ID of the suite.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}' f'/suites/{suite_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)

        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None



from testlodge.client.base import BaseClient
from testlodge.client.custom_field import CustomFieldClient
from testlodge.client.project import ProjectClient
from testlodge.client.user import UserClient


class Client(UserClient, ProjectClient, CustomFieldClient, BaseClient):
    ...


__all__ = ['Client']
